@@ -893,8 +893,7 @@ class EmergeLogPackageTask(EmergeLogTask):
         """
         return ") Merging " in line
 
-
-class EmergeLog:
+class EmergeLog(object):
     """
     EmergeLog(after).get_update(package_pattern)
     """
@@ -948,20 +947,15 @@ class EmergeLog:
         """
         Получить список пакетов
         """
-        self._list, self._remove_list = \
-            zip(*self._parse_log(self._get_last_changes()))
-        self._list = filter(None, self._list)
-        self._remove_list = filter(None, self._remove_list)
-
-    def _parse_log(self, data):
-        searcher = lambda x: (self.re_complete_emerge.search(x) or
-                              self.re_complete_unmerge.search(x))
-        for re_match in ifilter(None, imap(searcher, data)):
-            if re_match.group(1) == "emerge":
-                yield re_match.group(2), None
-            else:
-                yield None, re_match.group(2)
-        yield None, None
+        self._list = []
+        self._remove_list = []
+        for line in self._get_last_changes():
+            m = self.re_complete_emerge.search(line)
+            if m:
+                self._list.append(m.group(2))
+            m = self.re_complete_unmerge.search(line)
+            if m:
+                self._remove_list.append(m.group(2))
 
     def _set_marker(self, text_marker):
         with open(self.emerge_log, 'a') as f:
@@ -983,6 +977,40 @@ class EmergeLog:
         if marker:
             self._set_marker(marker)
 
+class EmergeLogFiltered(EmergeLog):
+    """
+    Возвращает только список пакетов установленных указанным образом
+    """
+    class EmergeType(object):
+        Source = "source"
+        Binary = "binary"
+
+    re_merge_type = re.compile(r"Merging (Binary )?\((.*?)::", re.M)
+    emerge_type = EmergeType.Source
+
+    def get_packages(self):
+        """
+        Получить список пакетов
+        """
+        merge_types = {}
+        self._list = []
+        self._remove_list = []
+        for line in self._get_last_changes():
+            if "Started emerge" in line:
+                merge_types = {}
+            m_type = self.re_merge_type.search(line)
+            if m_type:
+                if m_type.group(1) is None:
+                    merge_types[m_type.group(2)] = self.EmergeType.Source
+                else:
+                    merge_types[m_type.group(2)] = self.EmergeType.Binary
+            m = self.re_complete_emerge.search(line)
+            if m and merge_types.get(m.group(2),'') is self.emerge_type:
+                self._list.append(m.group(2))
+            m = self.re_complete_unmerge.search(line)
+            if m:
+                self._remove_list.append(m.group(2))
+
 
 class PackageList(object):
     """
